@@ -13,6 +13,8 @@ import {RegistryContext} from './Contexts';
 
 import styles from './ContextMenuItem.css';
 
+import type {RegistryContextType} from './Contexts';
+
 type Props = {|
   children: React$Node,
   onClick: () => void,
@@ -20,7 +22,7 @@ type Props = {|
 |};
 
 export default function ContextMenuItem({children, onClick, title}: Props) {
-  const {hideMenu} = useContext(RegistryContext);
+  const {hideMenu} = useContext<RegistryContextType>(RegistryContext);
 
   const handleClick = event => {
     onClick();
