@@ -11,33 +11,53 @@ import {createContext} from 'react';
 
 export type ShowFn = ({|data: Object, pageX: number, pageY: number|}) => void;
 export type HideFn = () => void;
+export type OnChangeFn = boolean => void;
 
 const idToShowFnMap = new Map<string, ShowFn>();
 const idToHideFnMap = new Map<string, HideFn>();
 
-let currentHideFn = null;
+let currentHide: ?HideFn = null;
+let currentOnChange: ?OnChangeFn = null;
 
 function hideMenu() {
-  if (typeof currentHideFn === 'function') {
-    currentHideFn();
+  if (typeof currentHide === 'function') {
+    currentHide();
+
+    if (typeof currentOnChange === 'function') {
+      currentOnChange(false);
+    }
   }
+
+  currentHide = null;
+  currentOnChange = null;
 }
 
 function showMenu({
   data,
   id,
+  onChange,
   pageX,
   pageY,
 }: {|
   data: Object,
   id: string,
+  onChange?: OnChangeFn,
   pageX: number,
   pageY: number,
 |}) {
   const showFn = idToShowFnMap.get(id);
   if (typeof showFn === 'function') {
-    currentHideFn = idToHideFnMap.get(id);
+    // Prevent open menus from being left hanging.
+    hideMenu();
+
+    currentHide = idToHideFnMap.get(id);
+
     showFn({data, pageX, pageY});
+
+    if (typeof onChange === 'function') {
+      currentOnChange = onChange;
+      onChange(true);
+    }
   }
 }
 
@@ -55,7 +75,13 @@ function registerMenu(id: string, showFn: ShowFn, hideFn: HideFn) {
   };
 }
 
-export const RegistryContext = createContext({
+export type RegistryContextType = {|
+  hideMenu: typeof hideMenu,
+  showMenu: typeof showMenu,
+  registerMenu: typeof registerMenu,
+|};
+
+export const RegistryContext = createContext<RegistryContextType>({
   hideMenu,
   showMenu,
   registerMenu,
