@@ -8,7 +8,5 @@
  */
 
 export const barWidthThreshold = 2;
-export const interactionCommitSize = 10;
-export const interactionLabelWidth = 200;
 export const maxBarWidth = 30;
 export const minBarWidth = 5;
