@@ -11,86 +11,12 @@
 
 let React = require('react');
 let ReactDOM = require('react-dom');
+let ReactFeatureFlags = require('shared/ReactFeatureFlags');
 let ReactTestUtils = require('react-dom/test-utils');
 
-/**
- * Counts clicks and has a renders an item for each click. Each item rendered
- * has a ref of the form "clickLogN".
- */
-class ClickCounter extends React.Component {
-  state = {count: this.props.initialCount};
-
-  triggerReset = () => {
-    this.setState({count: this.props.initialCount});
-  };
-
-  handleClick = () => {
-    this.setState({count: this.state.count + 1});
-  };
-
-  render() {
-    const children = [];
-    let i;
-    for (i = 0; i < this.state.count; i++) {
-      children.push(
-        <div
-          className="clickLogDiv"
-          key={'clickLog' + i}
-          ref={'clickLog' + i}
-        />,
-      );
-    }
-    return (
-      <span className="clickIncrementer" onClick={this.handleClick}>
-        {children}
-      </span>
-    );
-  }
-}
-
-/**
- * Only purpose is to test that refs are tracked even when applied to a
- * component that is injected down several layers. Ref systems are difficult to
- * build in such a way that ownership is maintained in an airtight manner.
- */
-class GeneralContainerComponent extends React.Component {
-  render() {
-    return <div>{this.props.children}</div>;
-  }
-}
-
-/**
- * Notice how refs ownership is maintained even when injecting a component
- * into a different parent.
- */
-class TestRefsComponent extends React.Component {
-  doReset = () => {
-    this.refs.myCounter.triggerReset();
-  };
-
-  render() {
-    return (
-      <div>
-        <div ref="resetDiv" onClick={this.doReset}>
-          Reset Me By Clicking This.
-        </div>
-        <GeneralContainerComponent ref="myContainer">
-          <ClickCounter ref="myCounter" initialCount={1} />
-        </GeneralContainerComponent>
-      </div>
-    );
-  }
-}
-
-const expectClickLogsLengthToBe = function(instance, length) {
-  const clickLogs = ReactTestUtils.scryRenderedDOMComponentsWithClass(
-    instance,
-    'clickLogDiv',
-  );
-  expect(clickLogs.length).toBe(length);
-  expect(Object.keys(instance.refs.myCounter.refs).length).toBe(length);
-};
-
+// This is testing if string refs are deleted from `instance.refs`
+// Once support for string refs is removed, this test can be removed.
+// Detaching is already tested in refs-detruction-test.js
 describe('reactiverefs', () => {
   let container;
 
@@ -98,6 +24,7 @@ describe('reactiverefs', () => {
     jest.resetModules();
     React = require('react');
     ReactDOM = require('react-dom');
+    ReactFeatureFlags = require('shared/ReactFeatureFlags');
     ReactTestUtils = require('react-dom/test-utils');
   });
 
@@ -108,13 +35,117 @@ describe('reactiverefs', () => {
     }
   });
 
+  /**
+   * Counts clicks and has a renders an item for each click. Each item rendered
+   * has a ref of the form "clickLogN".
+   */
+  class ClickCounter extends React.Component {
+    state = {count: this.props.initialCount};
+
+    triggerReset = () => {
+      this.setState({count: this.props.initialCount});
+    };
+
+    handleClick = () => {
+      this.setState({count: this.state.count + 1});
+    };
+
+    render() {
+      const children = [];
+      let i;
+      for (i = 0; i < this.state.count; i++) {
+        children.push(
+          <div
+            className="clickLogDiv"
+            key={'clickLog' + i}
+            ref={'clickLog' + i}
+          />,
+        );
+      }
+      return (
+        <span className="clickIncrementer" onClick={this.handleClick}>
+          {children}
+        </span>
+      );
+    }
+  }
+
+  const expectClickLogsLengthToBe = function(instance, length) {
+    const clickLogs = ReactTestUtils.scryRenderedDOMComponentsWithClass(
+      instance,
+      'clickLogDiv',
+    );
+    expect(clickLogs.length).toBe(length);
+    expect(Object.keys(instance.refs.myCounter.refs).length).toBe(length);
+  };
+
   /**
    * Render a TestRefsComponent and ensure that the main refs are wired up.
    */
   const renderTestRefsComponent = function() {
+    /**
+     * Only purpose is to test that refs are tracked even when applied to a
+     * component that is injected down several layers. Ref systems are difficult to
+     * build in such a way that ownership is maintained in an airtight manner.
+     */
+    class GeneralContainerComponent extends React.Component {
+      render() {
+        return <div>{this.props.children}</div>;
+      }
+    }
+
+    /**
+     * Notice how refs ownership is maintained even when injecting a component
+     * into a different parent.
+     */
+    class TestRefsComponent extends React.Component {
+      doReset = () => {
+        this.refs.myCounter.triggerReset();
+      };
+
+      render() {
+        return (
+          <div>
+            <div ref="resetDiv" onClick={this.doReset}>
+              Reset Me By Clicking This.
+            </div>
+            <GeneralContainerComponent ref="myContainer">
+              <ClickCounter ref="myCounter" initialCount={1} />
+            </GeneralContainerComponent>
+          </div>
+        );
+      }
+    }
+
     container = document.createElement('div');
     document.body.appendChild(container);
-    const testRefsComponent = ReactDOM.render(<TestRefsComponent />, container);
+
+    let testRefsComponent;
+    expect(() => {
+      testRefsComponent = ReactDOM.render(<TestRefsComponent />, container);
+    }).toErrorDev(
+      ReactFeatureFlags.warnAboutStringRefs
+        ? [
+            'Warning: Component "div" contains the string ref "resetDiv". ' +
+              'Support for string refs will be removed in a future major release. ' +
+              'We recommend using useRef() or createRef() instead. ' +
+              'Learn more about using refs safely here: https://reactjs.org/link/strict-mode-string-ref\n' +
+              '    in div (at **)\n' +
+              '    in TestRefsComponent (at **)',
+            'Warning: Component "span" contains the string ref "clickLog0". ' +
+              'Support for string refs will be removed in a future major release. ' +
+              'We recommend using useRef() or createRef() instead. ' +
+              'Learn more about using refs safely here: https://reactjs.org/link/strict-mode-string-ref\n' +
+              '    in span (at **)\n' +
+              '    in ClickCounter (at **)\n' +
+              '    in div (at **)\n' +
+              '    in GeneralContainerComponent (at **)\n' +
+              '    in div (at **)\n' +
+              '    in TestRefsComponent (at **)',
+          ]
+        : [],
+    );
+
     expect(testRefsComponent instanceof TestRefsComponent).toBe(true);
 
     const generalContainer = testRefsComponent.refs.myContainer;
@@ -156,13 +187,14 @@ describe('reactiverefs', () => {
   });
 });
 
-if (!require('shared/ReactFeatureFlags').disableModulePatternComponents) {
+if (!ReactFeatureFlags.disableModulePatternComponents) {
   describe('factory components', () => {
     it('Should correctly get the ref', () => {
       function Comp() {
         return {
+          elemRef: React.createRef(),
           render() {
-            return <div ref="elemRef" />;
+            return <div ref={this.elemRef} />;
           },
         };
       }
@@ -177,7 +209,7 @@ if (!require('shared/ReactFeatureFlags').disableModulePatternComponents) {
           '`Comp.prototype = React.Component.prototype`. ' +
           "Don't use an arrow function since it cannot be called with `new` by React.",
       );
-      expect(inst.refs.elemRef.tagName).toBe('DIV');
+      expect(inst.elemRef.current.tagName).toBe('DIV');
     });
   });
 }
@@ -191,10 +223,15 @@ describe('ref swapping', () => {
     jest.resetModules();
     React = require('react');
     ReactDOM = require('react-dom');
+    ReactFeatureFlags = require('shared/ReactFeatureFlags');
     ReactTestUtils = require('react-dom/test-utils');
 
     RefHopsAround = class extends React.Component {
       state = {count: 0};
+      hopRef = React.createRef();
+      divOneRef = React.createRef();
+      divTwoRef = React.createRef();
+      divThreeRef = React.createRef();
 
       moveRef = () => {
         this.setState({count: this.state.count + 1});
@@ -212,15 +249,15 @@ describe('ref swapping', () => {
           <div>
             <div
               className="first"
-              ref={count % 3 === 0 ? 'hopRef' : 'divOneRef'}
+              ref={count % 3 === 0 ? this.hopRef : this.divOneRef}
             />
             <div
               className="second"
-              ref={count % 3 === 1 ? 'hopRef' : 'divTwoRef'}
+              ref={count % 3 === 1 ? this.hopRef : this.divTwoRef}
             />
             <div
               className="third"
-              ref={count % 3 === 2 ? 'hopRef' : 'divThreeRef'}
+              ref={count % 3 === 2 ? this.hopRef : this.divThreeRef}
             />
           </div>
         );
@@ -244,28 +281,28 @@ describe('ref swapping', () => {
       'third',
     );
 
-    expect(refHopsAround.refs.hopRef).toEqual(firstDiv);
-    expect(refHopsAround.refs.divTwoRef).toEqual(secondDiv);
-    expect(refHopsAround.refs.divThreeRef).toEqual(thirdDiv);
+    expect(refHopsAround.hopRef.current).toEqual(firstDiv);
+    expect(refHopsAround.divTwoRef.current).toEqual(secondDiv);
+    expect(refHopsAround.divThreeRef.current).toEqual(thirdDiv);
 
     refHopsAround.moveRef();
-    expect(refHopsAround.refs.divOneRef).toEqual(firstDiv);
-    expect(refHopsAround.refs.hopRef).toEqual(secondDiv);
-    expect(refHopsAround.refs.divThreeRef).toEqual(thirdDiv);
+    expect(refHopsAround.divOneRef.current).toEqual(firstDiv);
+    expect(refHopsAround.hopRef.current).toEqual(secondDiv);
+    expect(refHopsAround.divThreeRef.current).toEqual(thirdDiv);
 
     refHopsAround.moveRef();
-    expect(refHopsAround.refs.divOneRef).toEqual(firstDiv);
-    expect(refHopsAround.refs.divTwoRef).toEqual(secondDiv);
-    expect(refHopsAround.refs.hopRef).toEqual(thirdDiv);
+    expect(refHopsAround.divOneRef.current).toEqual(firstDiv);
+    expect(refHopsAround.divTwoRef.current).toEqual(secondDiv);
+    expect(refHopsAround.hopRef.current).toEqual(thirdDiv);
 
     /**
      * Make sure that after the third, we're back to where we started and the
      * refs are completely restored.
      */
     refHopsAround.moveRef();
-    expect(refHopsAround.refs.hopRef).toEqual(firstDiv);
-    expect(refHopsAround.refs.divTwoRef).toEqual(secondDiv);
-    expect(refHopsAround.refs.divThreeRef).toEqual(thirdDiv);
+    expect(refHopsAround.hopRef.current).toEqual(firstDiv);
+    expect(refHopsAround.divTwoRef.current).toEqual(secondDiv);
+    expect(refHopsAround.divThreeRef.current).toEqual(thirdDiv);
   });
 
   it('always has a value for this.refs', () => {
@@ -309,7 +346,20 @@ describe('ref swapping', () => {
         return <div ref={1} />;
       }
     }
-    const a = ReactTestUtils.renderIntoDocument(<A />);
+    let a;
+    expect(() => {
+      a = ReactTestUtils.renderIntoDocument(<A />);
+    }).toErrorDev(
+      ReactFeatureFlags.warnAboutStringRefs
+        ? [
+            'Warning: Component "A" contains the string ref "1". ' +
+              'Support for string refs will be removed in a future major release. ' +
+              'We recommend using useRef() or createRef() instead. ' +
+              'Learn more about using refs safely here: https://reactjs.org/link/strict-mode-string-ref\n' +
+              '    in A (at **)',
+          ]
+        : [],
+    );
     expect(a.refs[1].nodeName).toBe('DIV');
   });
 
@@ -464,7 +514,7 @@ describe('root level refs', () => {
   });
 });
 
-describe('creating element with ref in constructor', () => {
+describe('creating element with string ref in constructor', () => {
   class RefTest extends React.Component {
     constructor(props) {
       super(props);
@@ -521,13 +571,41 @@ describe('strings refs across renderers', () => {
 
     const div1 = document.createElement('div');
     const div2 = document.createElement('div');
-    const inst = ReactDOM.render(<Parent />, div1);
+
+    let inst;
+    expect(() => {
+      inst = ReactDOM.render(<Parent />, div1);
+    }).toErrorDev(
+      ReactFeatureFlags.warnAboutStringRefs
+        ? [
+            'Warning: Component "Indirection" contains the string ref "child1". ' +
+              'Support for string refs will be removed in a future major release. ' +
+              'We recommend using useRef() or createRef() instead. ' +
+              'Learn more about using refs safely here: https://reactjs.org/link/strict-mode-string-ref\n' +
+              '    in Indirection (at **)\n' +
+              '    in Parent (at **)',
+          ]
+        : [],
+    );
+
     // Only the first ref has rendered yet.
     expect(inst.refs.child1.tagName).toBe('DIV');
     expect(inst.refs.child1).toBe(div1.firstChild);
 
-    // Now both refs should be rendered.
-    ReactDOM.render(<Parent />, div1);
+    expect(() => {
+      // Now both refs should be rendered.
+      ReactDOM.render(<Parent />, div1);
+    }).toErrorDev(
+      ReactFeatureFlags.warnAboutStringRefs
+        ? [
+            'Warning: Component "Root" contains the string ref "child2". ' +
+              'Support for string refs will be removed in a future major release. ' +
+              'We recommend using useRef() or createRef() instead. ' +
+              'Learn more about using refs safely here: https://reactjs.org/link/strict-mode-string-ref',
+          ]
+        : [],
+      {withoutStack: true},
+    );
     expect(inst.refs.child1.tagName).toBe('DIV');
     expect(inst.refs.child1).toBe(div1.firstChild);
     expect(inst.refs.child2.tagName).toBe('DIV');
