@@ -7,14 +7,11 @@
  * @flow
  */
 
-import invariant from 'shared/invariant';
-
 // Renderers that don't support test selectors
 // can re-export everything from this module.
 
 function shim(...args: any) {
-  invariant(
-    false,
+  throw new Error(
     'The current renderer does not support test selectors. ' +
       'This error is likely caused by a bug in React. ' +
       'Please file an issue.',
