@@ -8,6 +8,7 @@
  */
 
 import {REACT_PORTAL_TYPE} from 'shared/ReactSymbols';
+import {checkKeyStringCoercion} from 'shared/CheckStringCoercion';
 
 import type {ReactNodeList, ReactPortal} from 'shared/ReactTypes';
 
@@ -18,6 +19,9 @@ export function createPortal(
   implementation: any,
   key: ?string = null,
 ): ReactPortal {
+  if (__DEV__) {
+    checkKeyStringCoercion(key);
+  }
   return {
     // This tag allow us to uniquely identify this as a React Portal
     $$typeof: REACT_PORTAL_TYPE,
