@@ -10,19 +10,13 @@
 
 'use strict';
 
-let createRenderer;
-let PropTypes;
-let React;
+import * as PropTypes from 'prop-types';
+import * as React from 'react';
+import ReactShallowRenderer from 'react-test-renderer/shallow';
+
+const createRenderer = ReactShallowRenderer.createRenderer;
 
 describe('ReactShallowRenderer', () => {
-  beforeEach(() => {
-    jest.resetModules();
-
-    createRenderer = require('react-test-renderer/shallow').createRenderer;
-    PropTypes = require('prop-types');
-    React = require('react');
-  });
-
   it('should call all of the legacy lifecycle hooks', () => {
     const logs = [];
     const logger = message => () => logs.push(message) || true;
@@ -936,7 +930,7 @@ describe('ReactShallowRenderer', () => {
     let result = shallowRenderer.render(<SimpleComponent />);
     expect(result).toEqual(<div>value:0</div>);
 
-    let instance = shallowRenderer.getMountedInstance();
+    const instance = shallowRenderer.getMountedInstance();
     instance.updateState();
     result = shallowRenderer.getRenderOutput();
     expect(result).toEqual(<div>value:1</div>);
@@ -1176,7 +1170,7 @@ describe('ReactShallowRenderer', () => {
     }
 
     const shallowRenderer = createRenderer();
-    let result = shallowRenderer.render(<SimpleComponent />, {
+    const result = shallowRenderer.render(<SimpleComponent />, {
       foo: 'foo',
       bar: 'bar',
     });
@@ -1503,7 +1497,7 @@ describe('ReactShallowRenderer', () => {
     const shallowRenderer = createRenderer();
     shallowRenderer.render(<Foo foo={1} bar={1} />);
     expect(renderCount).toBe(1);
-    // Change a prop that the comparison funciton ignores
+    // Change a prop that the comparison function ignores
     shallowRenderer.render(<Foo foo={1} bar={2} />);
     expect(renderCount).toBe(1);
     shallowRenderer.render(<Foo foo={2} bar={2} />);
