@@ -8,54 +8,58 @@
  */
 
 export {
+  __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
+  act,
+  act as unstable_act,
   Children,
-  createRef,
   Component,
+  Fragment,
+  Profiler,
   PureComponent,
+  StrictMode,
+  Suspense,
+  SuspenseList,
+  SuspenseList as unstable_SuspenseList, // TODO: Remove once call sights updated to SuspenseList
+  cloneElement,
   createContext,
+  createElement,
+  createMutableSource,
+  createMutableSource as unstable_createMutableSource,
+  createRef,
+  createServerContext,
   forwardRef,
+  isValidElement,
   lazy,
   memo,
+  startTransition,
+  startTransition as unstable_startTransition, // TODO: Remove once call sights updated to startTransition
+  unstable_Cache,
+  unstable_DebugTracingMode,
+  unstable_LegacyHidden,
+  unstable_Offscreen,
+  unstable_Scope,
+  unstable_getCacheSignal,
+  unstable_getCacheForType,
+  unstable_useCacheRefresh,
+  useId,
   useCallback,
   useContext,
+  useDebugValue,
+  useDeferredValue,
+  useDeferredValue as unstable_useDeferredValue, // TODO: Remove once call sights updated to useDeferredValue
   useEffect,
   useImperativeHandle,
-  useDebugValue,
+  useInsertionEffect,
   useLayoutEffect,
   useMemo,
   useMutableSource,
   useMutableSource as unstable_useMutableSource,
-  createMutableSource,
-  createMutableSource as unstable_createMutableSource,
   useReducer,
   useRef,
   useState,
-  Fragment,
-  Profiler,
-  StrictMode,
-  Suspense,
-  unstable_LegacyHidden,
-  createElement,
-  cloneElement,
-  isValidElement,
-  version,
-  __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
-  // exposeConcurrentModeAPIs
+  useSyncExternalStore,
   useTransition,
-  useTransition as unstable_useTransition,
-  useDeferredValue,
-  useDeferredValue as unstable_useDeferredValue,
-  startTransition,
-  startTransition as unstable_startTransition,
-  SuspenseList,
-  SuspenseList as unstable_SuspenseList,
-  // enableBlocksAPI
-  block,
-  block as unstable_block,
-  // enableScopeAPI
-  unstable_Scope,
-  unstable_useOpaqueIdentifier,
-  // enableDebugTracing
-  unstable_DebugTracingMode,
+  useTransition as unstable_useTransition, // TODO: Remove once call sights updated to useTransition
+  version,
 } from './src/React';
 export {jsx, jsxs, jsxDEV} from './src/jsx/ReactJSX';
