@@ -37,7 +37,15 @@ export function memo<Props>(
       },
       set: function(name) {
         ownName = name;
-        if (type.displayName == null) {
+
+        // The inner component shouldn't inherit this display name in most cases,
+        // because the component may be used elsewhere.
+        // But it's nice for anonymous functions to inherit the name,
+        // so that our component-stack generation logic will display their frames.
+        // An anonymous function generally suggests a pattern like:
+        //   React.memo((props) => {...});
+        // This kind of inner function is not used elsewhere so the side effect is okay.
+        if (!type.name && !type.displayName) {
           type.displayName = name;
         }
       },
