@@ -114,7 +114,7 @@ describe('ReactElement', () => {
       render() {
         return (
           <div>
-            <Child ref="childElement" />
+            <Child ref={React.createRef()} />
           </div>
         );
       }
