@@ -19,13 +19,31 @@ import {
   REACT_SUSPENSE_LIST_TYPE,
   REACT_MEMO_TYPE,
   REACT_LAZY_TYPE,
-  REACT_FUNDAMENTAL_TYPE,
   REACT_SCOPE_TYPE,
-  REACT_BLOCK_TYPE,
-  REACT_SERVER_BLOCK_TYPE,
   REACT_LEGACY_HIDDEN_TYPE,
+  REACT_OFFSCREEN_TYPE,
+  REACT_CACHE_TYPE,
+  REACT_TRACING_MARKER_TYPE,
 } from 'shared/ReactSymbols';
-import {enableScopeAPI} from './ReactFeatureFlags';
+import {
+  enableScopeAPI,
+  enableCacheElement,
+  enableTransitionTracing,
+  enableDebugTracing,
+  enableLegacyHidden,
+  enableSymbolFallbackForWWW,
+} from './ReactFeatureFlags';
+
+let REACT_MODULE_REFERENCE;
+if (enableSymbolFallbackForWWW) {
+  if (typeof Symbol === 'function') {
+    REACT_MODULE_REFERENCE = Symbol.for('react.module.reference');
+  } else {
+    REACT_MODULE_REFERENCE = 0;
+  }
+} else {
+  REACT_MODULE_REFERENCE = Symbol.for('react.module.reference');
+}
 
 export default function isValidElementType(type: mixed) {
   if (typeof type === 'string' || typeof type === 'function') {
@@ -36,12 +54,15 @@ export default function isValidElementType(type: mixed) {
   if (
     type === REACT_FRAGMENT_TYPE ||
     type === REACT_PROFILER_TYPE ||
-    type === REACT_DEBUG_TRACING_MODE_TYPE ||
+    (enableDebugTracing && type === REACT_DEBUG_TRACING_MODE_TYPE) ||
     type === REACT_STRICT_MODE_TYPE ||
     type === REACT_SUSPENSE_TYPE ||
     type === REACT_SUSPENSE_LIST_TYPE ||
-    type === REACT_LEGACY_HIDDEN_TYPE ||
-    (enableScopeAPI && type === REACT_SCOPE_TYPE)
+    (enableLegacyHidden && type === REACT_LEGACY_HIDDEN_TYPE) ||
+    type === REACT_OFFSCREEN_TYPE ||
+    (enableScopeAPI && type === REACT_SCOPE_TYPE) ||
+    (enableCacheElement && type === REACT_CACHE_TYPE) ||
+    (enableTransitionTracing && type === REACT_TRACING_MARKER_TYPE)
   ) {
     return true;
   }
@@ -53,9 +74,12 @@ export default function isValidElementType(type: mixed) {
       type.$$typeof === REACT_PROVIDER_TYPE ||
       type.$$typeof === REACT_CONTEXT_TYPE ||
       type.$$typeof === REACT_FORWARD_REF_TYPE ||
-      type.$$typeof === REACT_FUNDAMENTAL_TYPE ||
-      type.$$typeof === REACT_BLOCK_TYPE ||
-      type[(0: any)] === REACT_SERVER_BLOCK_TYPE
+      // This needs to include all possible module reference object
+      // types supported by any Flight configuration anywhere since
+      // we don't know which Flight build this will end up being used
+      // with.
+      type.$$typeof === REACT_MODULE_REFERENCE ||
+      type.getModuleId !== undefined
     ) {
       return true;
     }
