@@ -1,7 +1,7 @@
 'use strict';
 
 jest.mock('react-noop-renderer', () =>
-  require.requireActual('react-noop-renderer/persistent')
+  jest.requireActual('react-noop-renderer/persistent')
 );
 
 global.__PERSISTENT__ = true;
